"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GithubGistApi = exports.getHelloWorldString = void 0;
const file_1 = require("./models/file");
const argument_not_specified_1 = require("./util/argument-not-specified");
const formatted_gist_identifier_1 = require("./util/formatted-gist-identifier");
const is_string_empty_1 = require("./util/is-string-empty");
const token_validity_1 = require("./token-validity");
const constants_1 = require("./constants");
const axios_1 = require("axios");
const getHelloWorldString = () => {
    return 'Hello World!';
};
exports.getHelloWorldString = getHelloWorldString;
class GithubGistApi {
    constructor(personalAccessToken = (0, argument_not_specified_1.default)('personalAccessToken'), _pureGistIdentifierName = (0, argument_not_specified_1.default)('gistIdentifierName')) {
        this.personalAccessToken = personalAccessToken;
        this._pureGistIdentifierName = _pureGistIdentifierName;
        this._gistId = '';
        this._username = '';
        this._files = [];
        this.isPublic = false;
        this.allowCors = false;
        this._gistIdentifierName = (0, formatted_gist_identifier_1.default)(_pureGistIdentifierName);
    }
    get gistId() {
        return this._gistId;
    }
    get username() {
        return this._username;
    }
    touch() {
        return __awaiter(this, void 0, void 0, function* () {
            yield (0, token_validity_1.default)(this.personalAccessToken);
            yield this._fetchGistIfExists();
            if ((0, is_string_empty_1.default)(this._gistId)) {
                yield this._createGistForFirstTime();
            }
        });
    }
    createFile(name, initialContent) {
        for (const file of this._files) {
            if (file.name === name) {
                throw new Error(`A file named ${name} already exists.`);
            }
        }
        const file = new file_1.default(this, name, initialContent);
        this._files.push(file);
    }
    getFile(name) {
        for (const file of this._files) {
            if (file.name === name) {
                return file;
            }
        }
        throw new Error('No file found with name: ' + name);
    }
    getFileNames() {
        return this._files.map(file => file.name);
    }
    save() {
        return __awaiter(this, void 0, void 0, function* () {
            const files = {};
            for (const file of this._files) {
                if (file.hasUpdates) {
                    files[file.name] = {
                        content: file.getContent()
                    };
                }
            }
            if (Object.keys(files).length === 0) {
                // All files are already updated
                return;
            }
            const url = `${constants_1.default.githubGists}/${this.gistId}`;
            const body = {
                public: this.isPublic,
                files
            };
            try {
                yield axios_1.default.post(url, body, this._authConfig());
            }
            catch (e) {
                throw new Error('One or more files couldn\'t be saved.');
            }
        });
    }
    _fetchGistIfExists() {
        return __awaiter(this, void 0, void 0, function* () {
            let result;
            try {
                result = yield axios_1.default.get(constants_1.default.githubGists, this._authConfig());
            }
            catch (e) {
                throw new Error('Error while fetching gists.');
            }
            const gists = result.data;
            for (const gist of gists) {
                const fileNames = Object.keys(gist.files);
                if (fileNames.includes(this._gistIdentifierName)) {
                    yield this._selfInitialize(gist);
                    break;
                }
            }
        });
    }
    _createGistForFirstTime() {
        return __awaiter(this, void 0, void 0, function* () {
            const rootFileContent = constants_1.default.identifier
                .content.replace('<APP-NAME>', this._pureGistIdentifierName);
            const data = {
                public: this.isPublic,
                files: {
                    [this._gistIdentifierName]: {
                        content: rootFileContent
                    }
                }
            };
            let result;
            try {
                result = yield axios_1.default.post(constants_1.default.githubGists, data, this._authConfig());
            }
            catch (e) {
                throw new Error('Error while creating the gist.');
            }
            yield this._selfInitialize(result.data);
        });
    }
    _getLatestCommit() {
        return __awaiter(this, void 0, void 0, function* () {
            const dummyParam = `dummyParam=${Math.random()}`;
            const url = `${constants_1.default.githubGists}/${this._gistId}?${dummyParam}`;
            try {
                const result = yield axios_1.default.get(url, this._authConfig());
                const response = result.data;
                const latestCommit = response.history[0];
                return latestCommit.version;
            }
            catch (e) {
                throw new Error('Error while fetching the latest commit.');
            }
        });
    }
    _selfInitialize(gist) {
        return __awaiter(this, void 0, void 0, function* () {
            this._gistId = gist.id;
            this._username = gist.owner.login;
            for (const fileName in gist.files) {
                if (gist.files.hasOwnProperty(fileName)) {
                    const file = new file_1.default(this, fileName, '');
                    yield file.fetchLatest();
                    this._files.push(file);
                }
            }
        });
    }
    _authConfig() {
        return {
            headers: { Authorization: `token ${this.personalAccessToken}` }
        };
    }
}
exports.GithubGistApi = GithubGistApi;
GithubGistApi.__getHelloWorldString = exports.getHelloWorldString;
